use crate::plausible::{Event, PlausibleAnalytics};
use clap::Parser;
use futures_util::io::AsyncBufReadExt;
use futures_util::TryStreamExt;
use k8s_openapi::api::core::v1::Pod;
use kube::api::{ListParams, LogParams};
use kube::client::ClientBuilder;
use kube::config::Kubeconfig;
use kube::{Api, Client, Config};
use log::{error, info};
use nginx_log_parser::Format;
use std::str::FromStr;

mod plausible;

#[derive(Parser)]
#[command(about)]
struct App {
    #[arg(long)]
    config: Option<String>,

    #[arg(long)]
    namespace: Option<String>,

    #[arg(long)]
    plausible: String,
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    pretty_env_logger::init();

    let app = App::parse();

    let namespace = if let Some(n) = app.namespace {
        n
    } else {
        "ingress-nginx".to_string()
    };

    let client = if let Some(c) = app.config {
        let cfg = Kubeconfig::read_from(c.as_str())?;
        let cfg = Config::from_custom_kubeconfig(cfg, &Default::default()).await?;
        ClientBuilder::try_from(cfg)?.build()
    } else {
        Client::try_default().await?
    };

    let api: Api<Pod> = Api::namespaced(client, &namespace);
    let pods = api
        .list(&ListParams::default().labels("app.kubernetes.io/component=controller"))
        .await?;
    info!("Found {} pods!", pods.items.len());

    let plausible = PlausibleAnalytics::new(&app.plausible);
    let mut tasks = Vec::new();
    for pod in pods.items {
        let api = api.clone();
        let name = if let Some(n) = pod.metadata.name {
            n.clone()
        } else {
            continue;
        };
        let sender = plausible.sender();
        let t = tokio::spawn(async move {
            if let Ok(log_stream) = api
                .log_stream(
                    &name,
                    &LogParams {
                        follow: true,
                        container: Some("controller".to_string()),
                        ..Default::default()
                    },
                )
                .await
            {
                let mut stream = log_stream.lines();
                let format = r#"$remote_addr - $remote_user [$time_local] "$request" $status $body_bytes_sent "$http_referer" "$http_user_agent" $request_length $request_time [$proxy_upstream_name] [$proxy_alternative_upstream_name] $upstream_addr $upstream_response_length $upstream_response_time $upstream_status $req_id"#;
                let format = Format::from_str(format).unwrap();
                while let Ok(Some(line)) = stream.try_next().await {
                    if let Some(entry) = format.parse(&line) {
                        let url = if let Some(r) = entry
                            .get("request")
                            .and_then(|r| r.split(" ").skip(1).next())
                        {
                            r
                        } else {
                            continue;
                        };

                        // assume tracking call
                        if url.ends_with("/api/event") {
                            continue;
                        }

                        sender
                            .send(Event {
                                name: "pageview".to_string(),
                                domain: entry
                                    .get("proxy_upstream_name")
                                    .unwrap_or("unknown")
                                    .to_string(),
                                url: url.to_string(),
                                referrer: entry.get("http_referer").map(|r| r.to_string()),
                                user_agent: entry.get("http_user_agent").map(|r| r.to_string()),
                                xff: entry.get("remote_addr").map(|r| r.to_string()),
                            })
                            .unwrap()
                    }
                }
            } else {
                error!("Failed to log pod {}", name);
            }
        });
        tasks.push(t);
    }

    for t in tasks {
        t.await?;
    }
    Ok(())
}
