use log::{info, warn};
use reqwest::ClientBuilder;
use serde::{Deserialize, Serialize};
use std::time::Duration;
use tokio::sync::mpsc::{unbounded_channel, UnboundedSender};

#[derive(Debug, Serialize, Deserialize)]
pub struct Event {
    pub name: String,
    pub domain: String,
    pub url: String,
    pub referrer: Option<String>,
    #[serde(skip_serializing)]
    pub user_agent: Option<String>,
    #[serde(skip_serializing)]
    pub xff: Option<String>,
}

pub struct PlausibleAnalytics {
    tx: UnboundedSender<Event>,
}

impl PlausibleAnalytics {
    pub fn new(url: &str) -> Self {
        let (tx, mut rx) = unbounded_channel::<Event>();

        let c = ClientBuilder::new().build().unwrap();
        let url = url.to_owned();
        tokio::spawn(async move {
            while let Some(msg) = rx.recv().await {
                let body = serde_json::to_string(&msg).unwrap();
                match c
                    .post(format!("{}/api/event", &url))
                    .header("User-Agent", msg.user_agent.unwrap_or("".to_string()))
                    .header("X-Forwarded-For", msg.xff.unwrap_or("".to_string()))
                    .header("content-type", "application/json")
                    .body(body)
                    .timeout(Duration::from_secs(30))
                    .send()
                    .await
                {
                    Ok(_v) => {
                        info!("sent event: {} {}", &msg.domain, &msg.url);
                    }
                    Err(e) => warn!("Failed to track: {}", e),
                }
            }
        });

        Self { tx }
    }

    pub fn sender(&self) -> UnboundedSender<Event> {
        self.tx.clone()
    }
}
